﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using rfid;

namespace TS_URW_DemoCS
{
  public partial class URW38Test : Form
  {
    private int myHandle;

    public URW38Test()
    {
      InitializeComponent();
      myHandle = 0;
      StartAddress.Text = "0";
      Length.Text = "4";
      MemoryBank.SelectedIndex = 3;
    }

    private void URW38Test_FormClosing(object sender, FormClosingEventArgs e)
    {
      CloseIfOpen();
    }

    bool OpenIfNeeded()
    {
      if (myHandle > 0)
      {
        // Device already opened
        return true;
      }

      if (DeviceList.SelectedIndex < 0)
        return false;

      string DeviceName = DeviceList.SelectedItem.ToString();
      if (DeviceName.Length > 0)
      {
        myHandle = GIS_URW_API.TSURW_Open(DeviceName, 19200, 0, 500);
        if (myHandle > 0)
        {
          DeviceList.Enabled = false;
          return true;
        }
      }
      return false;
    }

    void CloseIfOpen()
    {
      if (myHandle > 0)
        GIS_URW_API.TSURW_Close(myHandle);
      myHandle = 0;
      DeviceList.Enabled = true;
    }

    static string MakeStringFromArray(sbyte[] Arr, ref int StartIdx)
    {
      string strLine;
      strLine = "";
      int i;
      for (i = StartIdx; (i < Arr.Length) && (Arr[i] != 0); i++)
      {
        strLine += (char)Arr[i];
      }
      StartIdx = i+1;
      return strLine;
    }

    static string MakeStringFromData(byte[]Arr, int StartIdx, int EndIdx)
    {
      string strLine;
      strLine = "";
      int i;
      for (i = StartIdx; (i <= EndIdx); i++)
      {
        strLine += string.Format("{0:X2}",Arr[i]);
        strLine += " ";
      }
      StartIdx = i;
      return strLine;
    }

    static int MakeArrayFromString(sbyte[] Arr, string str)
    {
      int i;
      for (i = 0; i < str.Length; i++)
        Arr[i] = (SByte)str[i];
      Arr[i] = 0;
      return i;
    }

    static byte MakeHex(char high, char low)
    {
      string str = "";
      str += high;
      str += low;

      return Convert.ToByte(str,16);
    }

    static int MakeDataFromString(byte[] Arr, string str)
    {
      int i;
      int s1, s2;

      str.TrimStart(' ');
      str.TrimEnd(' ');
      s1 = 0; s2 = 0; i = 0;

      while ((i <= Arr.Length) && (s2<str.Length))
      {
        s1 = s2;
        while ((s2 < s1 + 2) && (s2<str.Length))
        {
          if (str[s2] == ' ')
            break;
          s2++;
        }
        if (s2 == s1 + 2)
          Arr[i++] = MakeHex(str[s1], str[s1 + 1]);
        else
          Arr[i++] = MakeHex(str[s1], '0');
        while ((s2<str.Length) && (str[s2] == ' '))
          s2++;
      }
      return i;
    }

    private void OnScan(object sender, EventArgs e)
    {
      int MaxLen = 255;
      int ListLen = 0;
      SByte[] NamenListe = new SByte[MaxLen];

      CloseIfOpen();

      DeviceList.Items.Clear();
      ListLen = GIS_URW_API.TSURW_GetUSBDeviceNames(NamenListe, MaxLen);

      if (ListLen > 1)
      {
        int StartIdx = 0;
        string Name;
        do
        {
          Name = MakeStringFromArray(NamenListe, ref StartIdx);
          if (Name.Length > 0)
            DeviceList.Items.Add(Name);
        }
        while (Name.Length > 0);
      }

      ListLen = GIS_URW_API.TSURW_GetCOMDeviceNames(NamenListe, MaxLen);
      if (ListLen > 1)
      {
        int StartIdx = 0;
        string Name;
        do
        {
          Name = MakeStringFromArray(NamenListe, ref StartIdx);
          if (Name.Length > 0)
            DeviceList.Items.Add(Name);

        }
        while (Name.Length > 0);
      }
      if (DeviceList.Items.Count > 0)
        DeviceList.SetSelected(0, true);
    }

    private void ReadInventory_Click(object sender, EventArgs e)
    {
      InventoryData.Text = "";

      if (OpenIfNeeded())
      {
        int BufLen = 100;
        int UsedLen;
        Byte[] Buffer = new Byte[BufLen];

        UsedLen = GIS_URW_API.TSURW_Inventory(myHandle, 0, Buffer, BufLen);
        if (UsedLen > 0)
        {
          InventoryData.Text = MakeStringFromData(Buffer, 3, UsedLen-1);
        }
      }
    }

    private void ReadMemory_Click(object sender, EventArgs e)
    {
      BlockData.Text = "";
      if (OpenIfNeeded())
      { 
        int UsedLen;
        int MemType;
        int StartIdx;
        int Len;
        int UIDLen;

        MemType = MemoryBank.SelectedIndex;
        StartIdx = Convert.ToInt32(StartAddress.Text);
        Len = Convert.ToInt32(Length.Text);
        Byte[] Buffer = new Byte[Len];

        Byte[] InventoryBuffer = new Byte[16];

        UIDLen = MakeDataFromString(InventoryBuffer, InventoryData.Text);
        if (GIS_URW_API.TSURW_Select(myHandle, InventoryBuffer, UIDLen) < 0)
          return;

        UsedLen = GIS_URW_API.TSURW_ReadData(myHandle, MemType,StartIdx/2,Buffer, Len);
        if (UsedLen > 0)
        {
          BlockData.Text = MakeStringFromData(Buffer, 0, UsedLen-1);
        }
      }
    }

    private void WriteMemory_Click(object sender, EventArgs e)
    {
      if (OpenIfNeeded())
      { 
        int UsedLen;
        int MemType;
        int StartIdx;
        int Len;
        int UIDLen;

        MemType = MemoryBank.SelectedIndex;
        StartIdx = Convert.ToInt32(StartAddress.Text);
        Len = Convert.ToInt32(Length.Text);
        Byte[] Buffer = new Byte[Len];

        Byte[] InventoryBuffer = new Byte[16];

        UIDLen = MakeDataFromString(InventoryBuffer, InventoryData.Text);
        if (GIS_URW_API.TSURW_Select(myHandle, InventoryBuffer, UIDLen) < 0)
          return;

        MakeDataFromString(Buffer, BlockData.Text);

        UsedLen = GIS_URW_API.TSURW_WriteData(myHandle, MemType,StartIdx/2,Buffer, Len);
        if (UsedLen >= 0)
          MessageBox.Show("Successfully written");
      }
    }
  }
}
