using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace rfid
{
  class GIS_URW_API
  {
		// Error codes DLL
    public const int TSURW_NO_ERROR                  = 0;
    public const int TSURW_PORT_OEFFNEN              = 1;
    public const int TSURW_TIMEOUT_UNGUELTIG         = 2;
    public const int TSURW_PORTHANDLE_UNGUELTIG      = 3;
    public const int TSURW_TIMEOUT                   = 4;
    public const int TSURW_BUFFER_ZU_KLEIN           = 5;
    public const int TSURW_BUFFERGROESSE_FALSCH      = 6;
    public const int TSURW_KEINE_SENDEDATEN          = 8;
    public const int TSURW_CHECKSUMMENFEHLER         = 10;
    public const int TSURW_KEINE_KOMMUNIKATION       = 11;
    public const int TSURW_CHECKBUFFER_LEER          = 12;
    public const int TSURW_MODE_UNGUELTIG            = 13;
    public const int TSURW_BEFEHL_NICHT_ERLAUBT      = 14;
    public const int TSURW_BLOCKNR_UNGUELTIG         = 15;

    public const int TSURW_DC3                       = 19; //0x13
    public const int TSURW_DC4                       = 20; //0x14
    public const int TSURW_NACK                      = 21; //0x15
    public const int TSURW_SYNC                      = 22; //0x16
    public const int TSURW_ETB                       = 23; //0x17
    public const int TSURW_CANCEL                    = 24; //0x18
    public const int TSURW_EM                        = 25; //0x19

		// Error codes Leser
    public const int TSURW_UNBEKANNTES_KOMMANDO      = 24;
    public const int TSURW_DATENANZAHL               = 20;
    public const int TSURW_KOLLISION                 = 23;
    public const int TSURW_NO_TAG                    = 21;
    public const int TSURW_NO_VALUE                  = 19;

		// Return values
    public const int TSURW_RETVAL_ERROR              = -1;
    public const int TSURW_RETVAL_OK                 = 0;

		//EPC Mem Banks
		// Definition for EPC tag memory bank: reserved
    public const int MEM_RES          = 0x00;
		// Definition for EPC tag memory bank: EPC memory
    public const int MEM_EPC          = 0x01;
		// Definition for EPC tag memory bank: TID
    public const int MEM_TID          = 0x02;
		// Definition for EPC tag memory bank: user
    public const int MEM_USER         = 0x03;

		// Definition of Inventory types
    public const int INVENTORY_NEW    = 0x00;
    public const int INVENTORY_START  = 0x01;
    public const int INVENTORY_NEXT   = 0x02;
    public const int INVENTORY_TID    = 0x03;

    public delegate int AutoReadCallback([MarshalAsAttribute(UnmanagedType.LPStr)] string pData, int Len);

		const string dllname = "TS_URW_SDK.dll";
    // Get Library Version
    [DllImport(dllname)] public static extern int TSURW_LibVersion();

    [DllImport(dllname)] public static extern int TSURW_IsVirtualComInstalled();

    // Get Names of all USB Devices connected, only valid devices are listed
    [DllImport(dllname)] public static extern int TSURW_GetUSBDeviceNames(SByte[] NamenListe, int nBufferSize);

    // Get Names of all USB Devices connected, all devices from GiS are listed
    [DllImport(dllname)] public static extern int TSURW_GetUSBDeviceNamesEx(SByte[] NamenListe, int nBufferSize);

    // Get Names of all COM Ports available, not tested if there is a device at the port connected.
    [DllImport(dllname)] public static extern int TSURW_GetCOMDeviceNames(SByte[] NamenListe, int nBufferSize);

    // Get Names of all valid LAN devices found in same subnet
    [DllImport(dllname)] public static extern int TSURW_GetLanDeviceNames(SByte[] NamenListe, int nBufferSize);

    // Get Names of all LAN devices found even in other subnets
    [DllImport(dllname)] public static extern int TSURW_GetLanDeviceNamesEx(SByte[] Buffer, int nBufferSize);

    // Change IP Address of LAN device
    [DllImport(dllname)] public static extern int TSURW_ChangeLanIPAddress(SByte[] OldAddress, SByte[] NewAddress, SByte[] IPMask);

    // Change IP Address and Gateway of LAN device
    [DllImport(dllname)] public static extern int TSURW_ChangeLanIPAddressEx(SByte[] OldAddress, SByte[] NewAddress, SByte[] IPMask, SByte[] GatewayAddress);

    // Get IP Address and Gateway of LAN device
    [DllImport(dllname)] public static extern int TSURW_GetLanIPAddressEx(SByte[] strName, SByte[] Address, SByte[] IPMask, SByte[] GatewayAddress);

    // Check if specific LAN device is available
    [DllImport(dllname)] public static extern int TSURW_IsLanDeviceAvailable(string Address);

    // Open LAN Device for configuration
    [DllImport(dllname)] public static extern int TSURW_LanConfigOpen(string strName);

    // Set Configuration Record of LAN device, use with care!
    [DllImport(dllname)] public static extern int TSURW_LanConfigGet(int nHandle, int RecordNr, Byte[] pData, int Len);

    // Get Configuration Record of LAN device
    [DllImport(dllname)] public static extern int TSURW_LanConfigSet(int nHandle, int RecordNr, Byte[] pData, int Len);

    // Close LAN device configuration.
    [DllImport(dllname)] public static extern int TSURW_LanConfigClose(int nHandle, int Reset);

    // Read Configuration of USB HID device
    [DllImport(dllname)] public static extern int TSURW_HIDConfigRead(int PortHandle, Byte[] pSendBuf, int SendBufLen, Byte[] pRecvBuf, int RecvBufLen);
    // Write Configuration to USB HID device
    [DllImport(dllname)] public static extern int TSURW_HIDConfigWrite(int PortHandle, Byte[] pSendBuf, int SendBufLen);

    [DllImport(dllname)] public static extern int TSURW_Open(string Name, int Baudrate, int ParityMode, int Timeout);
    [DllImport(dllname)] public static extern int TSURW_Close(int handle);
    [DllImport(dllname)] public static extern int TSURW_KeepAppActive(int handle, int bActive);

    [DllImport(dllname)] public static extern int TSURW_SetReaderAdresse(int handle, int Adresse);

    [DllImport(dllname)] public static extern int TSURW_StartAutoRead(int handle, int TermChar, AutoReadCallback cbFunction);
    [DllImport(dllname)] public static extern int TSURW_StopAutoRead(int handle);

    [DllImport(dllname)] public static extern int TSURW_GetDeviceVersion(int handle, Byte[] pDevVer, int VerLen, Byte[] pDevName, int NameLen);
    [DllImport(dllname)] public static extern int TSURW_IsUSB(int handle);
    [DllImport(dllname)] public static extern int TSURW_SetBaudrate(int handle, int Baudrate, int ParityMode);
    [DllImport(dllname)] public static extern int TSURW_SetReaderMode(int handle, int mode);
    [DllImport(dllname)] public static extern int TSURW_SetRF(int handle, int OnOff);
    [DllImport(dllname)] public static extern int TSURW_GetLastError(int handle);
    [DllImport(dllname)] public static extern int TSURW_SetIO(int handle, int Maske, int Daten);
    [DllImport(dllname)] public static extern int TSURW_ReadIO(int handle, int[] pDaten);
    [DllImport(dllname)] public static extern int TSURW_TestIO(int handle, int[] pTestwert);

    [DllImport(dllname)] public static extern int TSURW_SetLED(int handle, int red, int green, int yellow);
    [DllImport(dllname)] public static extern int TSURW_Beep(int handle);
		
    [DllImport(dllname)] public static extern int TSURW_ReadSerialNumber(int handle, Byte[] pSerial, int Buflen);
    [DllImport(dllname)] public static extern int TSURW_SetConfig(int handle, Byte[] pConfig, int ConfigLen);
    [DllImport(dllname)] public static extern int TSURW_GetConfig(int handle, Byte[] pConfig, int ConfigLen);
    [DllImport(dllname)] public static extern int TSURW_SetDefault(int handle);
    [DllImport(dllname)] public static extern int TSURW_SetCommunicationMode(int handle, int Mode);

    [DllImport(dllname)] public static extern int TSURW_RawWrite(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_RawRead(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_Transfer(int PortHandle, int Cmd, Byte[] pSendBuf, int SendBufLen, Byte[] pRecvBuf, int RecvBufLen);

		[DllImport(dllname)] public static extern int TSURW_GetTxPower    (int handle, int[]pVal);
		[DllImport(dllname)] public static extern int TSURW_SetTxPower    (int handle, int Val, int bPermanent);
    [DllImport(dllname)] public static extern int TSURW_ReadRegister  (int handle, int Register, int []pVal);
    [DllImport(dllname)] public static extern int TSURW_WriteRegister (int handle, int Register, int Val);
    [DllImport(dllname)] public static extern int TSURW_WriteRegister3(int handle, int Register, int Val);

    [DllImport(dllname)] public static extern int TSURW_Inventory(int PortHandle, int nInventoryType, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_InventoryRSSI(int PortHandle, int nInventoryType, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadData(int handle, int MemType, int nStartByte, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadAllData(int handle, int MemType, int nStartByte, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadDataPW (int handle, int MemType, int nStartByte, Byte[] pPassword, Byte[] pBuffer, int BufLen);
		[DllImport(dllname)] public static extern int TSURW_ReadAllDataPW (int handle, int MemType, int nStartWort, Byte[] pPassword, Byte[] pBuffer, int BufLen);

    [DllImport(dllname)] public static extern int TSURW_WriteData(int handle, int MemType, int nStartByte, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WriteDataPW(int handle, int MemType, int nStartByte, Byte[] pPassword, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_Select(int PortHandle, Byte[] pUID, int UIDLen);
		
    [DllImport(dllname)] public static extern int TSURW_Lock   (int handle, int MemType, int LockType, Byte[] pPassword);
    [DllImport(dllname)] public static extern int TSURW_KillTag(int handle, int RecomState, Byte[] pPassword);

    [DllImport(dllname)] 
    public static extern int TSURW_WriteParam(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadParam(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WritePrefix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadPrefix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WriteSuffix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadSuffix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WriteTermix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadTermix(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WritePostcode(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadPostcode(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_WriteReadModeParam(int handle, Byte[] pBuffer, int BufLen);
    [DllImport(dllname)] public static extern int TSURW_ReadReadModeParam(int handle, Byte[] pBuffer, int BufLen);
    
    public GIS_URW_API()
    {
    }
  }
}
